
h1 {
    font-size: 2.0em;
    margin-bottom: 1.5rem;
}

h2 {
    border-top: 2px solid #EBEBEB;
    border-radius: 0;
    margin-top: 3.0rem;
    margin-bottom: 1.0rem;
    padding-top: 1.5rem;
    font-size: 1.5em;
}

h3 {
    font-size: 1.2em;
    margin-top: 1.75rem;
}


.sidebar-brand-text {
    font-size: 1.4em;
}

.theme-toggle-container {
    display: none;
}

.toctree-wrapper {
    margin-top: 4.0rem;
}

code.docutils span {
    color: #941100;
    font-weight: 800;
}

section#version-history h2 {
    font-size: 1.2em;
}
